package jrummikub.view;

import jrummikub.util.GameData;
import jrummikub.util.IEvent;
import jrummikub.util.IEvent1;

/**
 * Panel showing all offered games in the chosen channel
 */
public interface IGameListPanel {
	/**
	 * Emitted when a new game is offered
	 * 
	 * @return the event
	 */
	public IEvent getOpenNewGameEvent();

	/**
	 * Emitted when the network game is canceled
	 * 
	 * @return the event
	 */
	public IEvent getCancelEvent();

	/**
	 * Emitted when the user chose to join an existing, open game
	 * 
	 * @return the event
	 */
	public IEvent1<GameData> getJoinEvent();

	/**
	 * Emitted when an open game is removed by the host
	 * 
	 * @param game
	 *            game data of the open game
	 */
	public void removeGame(GameData game);

	/**
	 * Adds a game to the list of open games
	 * 
	 * @param game
	 *            game data of the new game
	 */
	public void addGame(GameData game);

	/**
	 * Sets the channel name
	 * 
	 * @param name
	 *            channel name
	 */
	public void setChannelName(String name);
}
